from typing import Any

# ========== 前景色（字体颜色） ==========
def black(text: Any) -> str: ...
def red(text: Any) -> str: ...
def green(text: Any) -> str: ...
def yellow(text: Any) -> str: ...
def blue(text: Any) -> str: ...
def magenta(text: Any) -> str: ...
def cyan(text: Any) -> str: ...
def white(text: Any) -> str: ...

# 亮色 / 高亮前景
def bright_black(text: Any) -> str: ...
def bright_red(text: Any) -> str: ...
def bright_green(text: Any) -> str: ...
def bright_yellow(text: Any) -> str: ...
def bright_blue(text: Any) -> str: ...
def bright_magenta(text: Any) -> str: ...
def bright_cyan(text: Any) -> str: ...
def bright_white(text: Any) -> str: ...

# ========== 背景色 ==========
def bg_black(text: Any) -> str: ...
def bg_red(text: Any) -> str: ...
def bg_green(text: Any) -> str: ...
def bg_yellow(text: Any) -> str: ...
def bg_blue(text: Any) -> str: ...
def bg_magenta(text: Any) -> str: ...
def bg_cyan(text: Any) -> str: ...
def bg_white(text: Any) -> str: ...

# 亮背景
def bg_bright_black(text: Any) -> str: ...
def bg_bright_red(text: Any) -> str: ...
def bg_bright_green(text: Any) -> str: ...
def bg_bright_yellow(text: Any) -> str: ...
def bg_bright_blue(text: Any) -> str: ...
def bg_bright_magenta(text: Any) -> str: ...
def bg_bright_cyan(text: Any) -> str: ...
def bg_bright_white(text: Any) -> str: ...

# ========== 样式 ==========
def bold(text: Any) -> str: ...
def dim(text: Any) -> str: ...
def italic(text: Any) -> str: ...
def underline(text: Any) -> str: ...
def blink(text: Any) -> str: ...
def reverse(text: Any) -> str: ...
def hidden(text: Any) -> str: ...

# ========== 常用别名 ==========
def gray(text: Any) -> str: ...
def purple(text: Any) -> str: ...
def pink(text: Any) -> str: ...
def orange(text: Any) -> str: ...

def rgb2hex(r : int, g : int, b : int):
    """
    Convert RGB values to HEX values

    将 RGB 值转换为 HEX 值
    """
    ...
def hex2rgb(hex_str):
    """
    Convert HEX values to RGB values

    将 HEX 值转换为 RGB 值
    """
    ...